@@ -9,3 +9,6 @@
 #undef HAVE_LC_MESSAGES
 #undef HAVE_STPCPY
 #undef HAVE_PAM
+#undef HAVE_TCPWRAPPERS
+#undef HAVE_SHADOW
+
