@@ -33,21 +33,21 @@
 
 #include "gdm.h"
 
-static const gchar RCSid[]="$Id: gdmauth.c,v 1.9 1999/01/27 22:11:23 mkp Exp $";
+static const gchar RCSid[]="$Id: gdmauth.c,v 1.11 1999/02/06 01:14:50 mkp Exp $";
 
-extern gchar *gdmAuthDir;
-extern gid_t gdmGroupId;
+extern gchar *GdmAuthDir;
+extern gid_t GdmGroupId;
 
 extern gchar **gdm_arg_munch(gchar *);
-extern gchar *gdm_cookie_generate(void);
-extern void  *gdm_debug(const gchar *, ...);
+extern void gdm_cookie_generate(GdmDisplay *d);
+extern void *gdm_debug(const gchar *, ...);
 
-void gdm_auth_secure_display(gdmDisplayType *d);
-void gdm_auth_user_add(gdmDisplayType *d, gchar *home);
-void gdm_auth_user_remove(gdmDisplayType *d, gchar *home);
+void gdm_auth_secure_display(GdmDisplay *d);
+void gdm_auth_user_add(GdmDisplay *d, gchar *home);
+void gdm_auth_user_remove(GdmDisplay *d, gchar *home);
 
 
-void gdm_auth_secure_display(gdmDisplayType *d)
+void gdm_auth_secure_display(GdmDisplay *d)
 {
     gchar *authstr;
     gchar **argv;
@@ -55,42 +55,40 @@ void gdm_auth_secure_display(gdmDisplayType *d)
 
     gdm_debug(_("gdm_auth_secure_display: Securing %s"), d->name);
 
-    if(d->type==DISPLAY_LOCAL) {
+    gdm_cookie_generate(d);
 
-	d->cookie=gdm_cookie_generate();
-	d->auth=g_strconcat(gdmAuthDir, "/", d->name, ".xauth", NULL);
+    d->auth=g_strconcat(GdmAuthDir, "/", d->name, ".xauth", NULL);
+    
+    if(unlink(d->auth) == -1)
+	syslog(LOG_ERR, _("gdm_auth_secure_display: Could not unlink %s file: %s"),\
+	       d->auth, strerror(errno));
+    
+    authstr=g_strconcat(GDM_XAUTH_PATH, " -i -f ", d->auth, \
+			" add ", d->name, " . ", d->cookie, NULL);
+    argv=gdm_arg_munch(authstr);
+    
+    switch(authpid=fork()) {
+	
+    case 0:
+	execv(argv[0], argv);
+	syslog(LOG_ERR, _("gdm_auth_secure_display: Error starting xauth process: %s. Running insecure!"), strerror(errno));
+	return;
+	
+    case -1:
+	syslog(LOG_ERR, _("gdm_auth_secure_display: Error forking xauth process. Running insecure!"));
+	return;
+	
+    default:
+	waitpid(authpid, 0, 0); /* Wait for xauth to finish */
+	chown(d->auth, 0, GdmGroupId);
+	chmod(d->auth, S_IRUSR|S_IWUSR|S_IRGRP);
+	break;
+    }
 
-	if(unlink(d->auth) == -1)
-	    syslog(LOG_ERR, _("gdm_auth_secure_display: Could not unlink %s file: %s"),\
-		   d->auth, strerror(errno));
-
-	authstr=g_strconcat(GDM_XAUTH_PATH, " -i -f ", d->auth, \
-			       " add ", d->name, " . ", d->cookie, NULL);
-	argv=gdm_arg_munch(authstr);
-
-	switch(authpid=fork()) {
-
-	case 0:
-	    execv(argv[0], argv);
-	    syslog(LOG_ERR, _("gdm_auth_secure_display: Error starting xauth process: %s. Running insecure!"), strerror(errno));
-	    return;
-	    
-	case -1:
-	    syslog(LOG_ERR, _("gdm_auth_secure_display: Error forking xauth process. Running insecure!"));
-	    return;
-
-	default:
-	    waitpid(authpid, 0, 0); /* Wait for xauth to finish */
-	    chown(d->auth, 0, gdmGroupId);
-	    chmod(d->auth, S_IRUSR|S_IWUSR|S_IRGRP);
-	    break;
-	};
-
-    };
-};
+}
 
 
-void gdm_auth_user_add(gdmDisplayType *d, gchar *home)
+void gdm_auth_user_add(GdmDisplay *d, gchar *home)
 {
     gchar *authfile, *authstr;
     gchar **argv;
@@ -122,11 +120,11 @@ void gdm_auth_user_add(gdmDisplayType *d, gchar *home)
 	g_free(authfile);
 	break;
 
-    };    
-};
+    }
+}
 
 
-void gdm_auth_user_remove(gdmDisplayType *d, gchar *home)
+void gdm_auth_user_remove(GdmDisplay *d, gchar *home)
 {
     gchar *authstr;
     gchar **argv;
@@ -154,7 +152,7 @@ void gdm_auth_user_remove(gdmDisplayType *d, gchar *home)
 	waitpid(authpid, 0, 0); /* Wait for xauth to finish */
 	break;
 
-    };    
-};
+    }
+}
 
 /* EOF */
