@@ -31,12 +31,11 @@
 #include <fcntl.h>
 #include <sys/time.h>
 #include <unistd.h>
-#include <glib.h>
+#include <gnome.h>
 
+#include "gdm.h"
 #include "md5.h"
 
-static const gchar RCSid[]="$Id: gdmcookie.c,v 1.3 1999/01/27 22:11:25 mkp Exp $";
-
 #define MAXBUFFERSIZE 512
 
 struct rngs {
@@ -52,10 +51,10 @@ struct rngs {
 
 #define RNGS (sizeof(rngs)/sizeof(struct rngs))
 
-char* gdm_cookie_generate(void);
+void gdm_cookie_generate(GdmDisplay *);
 
-
-char* gdm_cookie_generate(void)
+void 
+gdm_cookie_generate(GdmDisplay *d)
 {
     int i;
     struct MD5Context ctx;
@@ -89,17 +88,19 @@ char* gdm_cookie_generate(void)
 		r = 0;
 	    close( fd );
 	    if (r >= rngs[i].length) break;
-	};
-    };
+	}
+    }
     
-    MD5Final( digest, &ctx );
+    MD5Final(digest, &ctx);
 
     for (i = 0; i < 16; i++) {
 	sprintf(sub, "%02x", digest[i]);
 	strcat(cookie, sub);
-    };
+    }
 
-    return ((char *)strdup(cookie));
+    d->cookie=g_strdup(cookie);
+    d->bcookie=g_strndup(digest, 16);
 }
 
+
 /* EOF */
