@@ -41,7 +41,7 @@
 
 #include "gdmgreeter.h"
 
-static const gchar RCSid[]="$Id: gdmgreeter.c,v 1.48 1999/01/27 22:11:26 mkp Exp $";
+static const gchar RCSid[]="$Id: gdmgreeter.c,v 1.54 1999/02/26 00:20:26 mkp Exp $";
 
 extern gchar * gdm_parse_enriched_string(gchar *str);
 extern gboolean gdm_file_check(gchar *caller, uid_t user, gchar *dir, gchar *file, gboolean absentok);
@@ -51,35 +51,37 @@ gchar *gdm_greeter_language_lookup(gchar *language);
 static gdmUserType *gdm_greeter_user_alloc (gchar *logname, gint uid, gchar *homedir);
 
 
-gint gdmNumberOfCols;
-gint gdmDisplayLogo;
-gint gdmDisplayBrowser;
-gint gdmShutdownMenu;
-gint gdmQuiver;
-gint gdmCurrentUser;
-gint gdmIconify;
-gint gdmUserMaxFile;
-gint gdmIconMaxWidth;
-gint gdmIconMaxHeight;
-gchar *gdmSuspend;
-gchar *gdmGlobalImageDir;
-gchar *gdmLogoFilename;
-gchar *gdmMessageText;
-gchar *gdmNofaceImageFile;
-gchar *gdmConfigFilename=GDM_CONFIG_FILE;
-gchar *gdmSessionDir;
-gchar *gdmIconFile;
-gchar *gdmMessageFont;
-gchar *gdmLocaleFile;
-gchar *gdmDefaultLocale;
-gchar *gdmPidFile;		/* Hack, not used in greeter */
+gint  GdmCompletion;
+gint  GdmDebug;
+gint  GdmDisplayBrowser;
+gint  GdmDisplayLogo;
+gint  GdmIconMaxHeight;
+gint  GdmIconMaxWidth;
+gint  GdmIconify;
+gint  GdmNumberOfCols;
+gint  GdmQuiver;
+gint  GdmShutdownMenu;
+gint  GdmUserMaxFile;
+gchar *GdmSuspend;
+gchar *GdmGlobalImageDir;
+gchar *GdmLogoFilename;
+gchar *GdmMessageText;
+gchar *GdmNofaceImageFile;
+gchar *GdmConfigFilename=GDM_CONFIG_FILE;
+gchar *GdmSessionDir;
+gchar *GdmIconFile;
+gchar *GdmMessageFont;
+gchar *GdmLocaleFile;
+gchar *GdmDefaultLocale;
+gchar *GdmPidFile;		/* Hack, not used in greeter */
+gchar *GdmDefaultPath;		/* Hack Part II */
+gchar *GdmGtkRC;
 
 GtkWidget *greeterframe;
 GtkWidget *gdmMain;
 GtkWidget *logobox;
 GtkWidget *msgbox;
 GtkWidget *msglabel;
-
 GtkWidget *loginentry;
 GtkWidget *passwdentry;
 GtkWidget *greeter;
@@ -92,6 +94,7 @@ GtkWidget *sessmenu;
 GtkWidget *langmenu;
 GtkWidget *loginbutton;
 GtkWidget *optionbutton;
+GtkWidget *browser_hbox;
 
 gchar *login, *passwd;
 gchar *defsess = NULL;
@@ -100,19 +103,19 @@ gchar *cursess = NULL;
 gchar *curlang = NULL;
 gchar *usrsess = NULL;
 gchar *usrlang = NULL;
+
 gboolean sessmatch=FALSE;
 gboolean langmatch=TRUE;
 gboolean sessmod=FALSE;
 gboolean langmod=TRUE;
 
-GnomeIconList *browser;
-GdkImlibImage *gdmNofaceImage;
-GtkWidget     *browser_hbox;
-GList *users;
-GList *result;
 GCompletion *cmpl;
-GSList *sessions=NULL;
+GList *result;
+GList *users;
 GSList *languages=NULL;
+GSList *sessions=NULL;
+GdkImlibImage *nofaceimg;
+GnomeIconList *browser;
 
 
 
@@ -166,14 +169,14 @@ gdm_greeter_message_init(void)
     GtkStyle *style;
     gchar *msg=NULL;
 
-    msg=gdm_parse_enriched_string(gdmMessageText);
+    msg=gdm_parse_enriched_string(GdmMessageText);
 
     msgbox = gtk_vbox_new(FALSE, 0);
 
     if(msg) {	
 	style = gtk_style_new();
 	gdk_font_unref(style->font);
-	style->font = gdk_font_load (gdmMessageFont);
+	style->font = gdk_font_load (GdmMessageFont);
 	gtk_widget_push_style(style);
 
 	msglabel = gtk_label_new(msg);
@@ -191,16 +194,12 @@ gdm_greeter_logo_init (void)
 {
     GtkWidget *logo;
     GtkWidget *frame;
-    GdkPixmap *pixmap;
-    GdkBitmap *mask;
 
-    if(access(gdmLogoFilename, R_OK)) {
+    if(access(GdmLogoFilename, R_OK)) {
 	syslog(LOG_WARNING, _("Logo not found. No image will be displayed!"));
     }
     else {
-	gdk_imlib_load_file_to_pixmap(gdmLogoFilename, &pixmap, &mask);
-	
-	logo = gtk_pixmap_new(pixmap, mask);	
+	logo = gnome_pixmap_new_from_file (GdmLogoFilename);
 	gtk_widget_show(logo);
 	
 	frame = gtk_frame_new(NULL);
@@ -220,39 +219,52 @@ static void
 gdm_greeter_parse_config(void)
 {
     struct stat unused;
+    gchar *display;
 	
     if(stat(GDM_CONFIG_FILE, &unused) == -1)
 	gdm_greeter_abort(_("gdm_greeter_parse_config: No configuration file: %s. Aborting."), GDM_CONFIG_FILE);
 
     gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
 
-    gdmDisplayBrowser=gnome_config_get_int("appearance/Browser=1");
-    gdmDisplayLogo=gnome_config_get_int("appearance/Logo=1");
-    gdmNumberOfCols=gnome_config_get_int("appearance/BrowserColumns=4");
-    gdmLogoFilename=gnome_config_get_string("appearance/LogoImage");
-    gdmQuiver=gnome_config_get_int("appearance/Quiver=1");
-    gdmIconify=gnome_config_get_int("appearance/iconify=1");
-    gdmIconFile=gnome_config_get_string("appearance/iconfile=gdm.xpm");
-    gdmNofaceImageFile= gnome_config_get_string("appearance/NoFaceImage");
-    gdmGlobalImageDir=gnome_config_get_string("appearance/GlobalImageDir");
-    gdmMessageFont=gnome_config_get_string("appearance/msgfont=-adobe-helvetica-bold-r-normal-*-*-180-*-*-*-*-*-*");
+    GdmCompletion=gnome_config_get_int("appearance/Completion=1");
+    GdmDisplayBrowser=gnome_config_get_int("appearance/Browser=1");
+    GdmDisplayLogo=gnome_config_get_int("appearance/Logo=1");
+    GdmLogoFilename=gnome_config_get_string("appearance/LogoImage");
+    GdmQuiver=gnome_config_get_int("appearance/Quiver=1");
+    GdmIconify=gnome_config_get_int("appearance/iconify=1");
+    GdmIconFile=gnome_config_get_string("appearance/iconfile=gdm.xpm");
+    GdmNofaceImageFile= gnome_config_get_string("appearance/NoFaceImage");
+    GdmGlobalImageDir=gnome_config_get_string("appearance/GlobalImageDir");
+    GdmMessageFont=gnome_config_get_string("appearance/msgfont=-adobe-helvetica-bold-r-normal-*-*-180-*-*-*-*-*-*");
+    GdmGtkRC=gnome_config_get_string("appearance/gtkrc");
 
-    gdmShutdownMenu=gnome_config_get_int("system/ShutdownMenu=0");
-    gdmUserMaxFile=gnome_config_get_int("system/UserFileCutoffSize=65536");
-    gdmIconMaxWidth=gnome_config_get_int("system/UserIconMaxWidth=128");
-    gdmIconMaxHeight=gnome_config_get_int("system/UserIconMaxHeight=128");
-    gdmSuspend=g_strdup(gnome_config_get_string("system/SuspendCommand"));
-    gdmLocaleFile=g_strdup(gnome_config_get_string("system/LocaleFile=/usr/share/locale/locale.alias"));
-    gdmDefaultLocale=g_strdup(gnome_config_get_string("system/DefaultLocale=english"));
+    GdmShutdownMenu=gnome_config_get_int("system/ShutdownMenu=0");
+    GdmUserMaxFile=gnome_config_get_int("system/UserFileCutoffSize=65536");
+    GdmIconMaxWidth=gnome_config_get_int("system/UserIconMaxWidth=128");
+    GdmIconMaxHeight=gnome_config_get_int("system/UserIconMaxHeight=128");
+    GdmSuspend=g_strdup(gnome_config_get_string("system/SuspendCommand"));
+    GdmLocaleFile=g_strdup(gnome_config_get_string("system/LocaleFile=/usr/share/locale/locale.alias"));
+    GdmDefaultLocale=g_strdup(gnome_config_get_string("system/DefaultLocale=english"));
 
-    gdmMessageText=gnome_config_get_string(_("messages/welcome=Welcome to %h"));
+    GdmMessageText=gnome_config_get_string(_("messages/welcome=Welcome to %h"));
 
-    gdmSessionDir=gnome_config_get_string("daemon/sessiondir");
+    GdmSessionDir=gnome_config_get_string("daemon/sessiondir");
+
+    GdmDebug=gnome_config_get_int("debug/enable=0");
 
     gnome_config_pop_prefix ();
 
-    if(stat(gdmLocaleFile, &unused) == -1)
-	gdm_greeter_abort("gdm_greeter_parse_config: Could not open locale file %s. Aborting!", gdmLocaleFile);
+    if(stat(GdmLocaleFile, &unused) == -1)
+	gdm_greeter_abort("gdm_greeter_parse_config: Could not open locale file %s. Aborting!", GdmLocaleFile);
+
+    /* Disable System menu on non-local displays */
+    display=getenv("DISPLAY");
+
+    if(!display)
+	gdm_greeter_abort("gdm_greeter_parse_config: DISPLAY variable not set!");
+
+    if(strncmp(display, ":", 1))
+	GdmShutdownMenu=0;
 }
 
 
@@ -277,7 +289,6 @@ gdm_greeter_info_handler(GtkWidget *widget, GtkWidget *dialog)
 {
     GdkCursor *cursor;
 
-    g_print("foo");
     cursor=gdk_cursor_new(GDK_LEFT_PTR);
     gdk_window_set_cursor (gdmMain->window, cursor);
     gdk_cursor_destroy(cursor);
@@ -401,7 +412,7 @@ gdm_greeter_invalid_passwd_req (void)
     int x,y;
 
     /* FIXME: esound! */
-    if(gdmQuiver) {
+    if(GdmQuiver) {
 	gdk_window_get_position(greeter->window, &x, &y);
 
 	for (i=32 ; i > 0 ; i=i/2) {
@@ -543,8 +554,6 @@ gdm_greeter_iconify_handler(GtkWidget *widget, gpointer data)
     GtkWidget *icon;
     GdkGC *gc;
     GtkStyle *style;
-    GdkPixmap *pixmap;
-    GdkBitmap *mask;
     CursorOffset *icon_pos;
     gint rw, rh, iw, ih;
 
@@ -565,13 +574,11 @@ gdm_greeter_iconify_handler(GtkWidget *widget, gpointer data)
     gtk_container_add(GTK_CONTAINER (win), fixed);
     gtk_widget_show(fixed);
 
-    gdk_imlib_load_file_to_pixmap(gdmIconFile, &pixmap, &mask);
-    gdk_window_get_size ((GdkWindow *) pixmap, &iw, &ih);
-
-    icon=gtk_pixmap_new(pixmap, mask);
+    icon=gnome_pixmap_new_from_file (GdmIconFile);
+    gdk_window_get_size ((GdkWindow *) GNOME_PIXMAP (icon)->pixmap, &iw, &ih);
+    
     gtk_fixed_put(GTK_FIXED (fixed), icon, 0, 0);
     gtk_widget_show(icon);
-    gtk_widget_shape_combine_mask(win, mask, 0, 0);
 
     gtk_signal_connect(GTK_OBJECT (win), "button_press_event",
 		       GTK_SIGNAL_FUNC (gdm_greeter_icon_pressed),NULL);
@@ -699,10 +706,8 @@ gdm_greeter_session_select(GtkWidget *widget, gchar *sess)
 {
     gchar *temp;
 
-    if(!widget || !sess) {
-	g_print("Mice!");
+    if(!widget || !sess) 
 	return;
-    }
 
     gtk_label_get(GTK_LABEL(GTK_BIN(widget)->child), &temp);
 
@@ -729,7 +734,7 @@ gdm_greeter_session_init(GtkWidget *parent)
     GtkWidget *submenu, *item;
 
     /* Check that session dir is readable */
-    if (access(gdmSessionDir, R_OK|X_OK))
+    if (access(GdmSessionDir, R_OK|X_OK))
 	gdm_greeter_abort(_("gdm_greeter_session_init: Session script directory not found!"));
 
     /* Create submenu */
@@ -737,13 +742,13 @@ gdm_greeter_session_init(GtkWidget *parent)
     gtk_menu_item_set_submenu(GTK_MENU_ITEM(parent), submenu);
 
     /* Read directory entries in session dir */
-    sessdir=opendir(gdmSessionDir);
+    sessdir=opendir(GdmSessionDir);
     dent=readdir(sessdir);
 
     while(dent != NULL) {
 	gchar *s;
 
-	s=g_strconcat(gdmSessionDir, "/", dent->d_name, NULL);
+	s=g_strconcat(GdmSessionDir, "/", dent->d_name, NULL);
 	lstat(s, &statbuf);
 
 	/* If default session link exists, find out what it points to */
@@ -774,7 +779,7 @@ gdm_greeter_session_init(GtkWidget *parent)
 	    }
 	    else 
 		syslog(LOG_ERR, "Wrong permissions on %s/%s. Should be readable/executable for all.", 
-		       gdmSessionDir, dent->d_name);
+		       GdmSessionDir, dent->d_name);
 
 	}
 
@@ -825,10 +830,8 @@ gdm_greeter_language_select(GtkWidget *widget, gchar *lang)
 {
     gchar *temp;
 
-    if(!widget || !lang) {
-	g_print("Rats!\n");
+    if(!widget || !lang)
 	return;
-    };
 
     gtk_label_get(GTK_LABEL(GTK_BIN(widget)->child), &temp);
 
@@ -853,12 +856,12 @@ gdm_greeter_language_init(GtkWidget *parent)
     char curline[256];
     char *ctmp, *ctmp1, *ctmp2;
 
-    deflang=gdmDefaultLocale;
+    deflang=GdmDefaultLocale;
     
     submenu = gtk_menu_new();
     gtk_menu_item_set_submenu(GTK_MENU_ITEM(parent), submenu);
 
-    langlist = fopen(gdmLocaleFile, "r");
+    langlist = fopen(GdmLocaleFile, "r");
     
     item = gtk_radio_menu_item_new_with_label(NULL, "English");
     languages = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(item));
@@ -972,7 +975,7 @@ gdm_greeter_buttons_init(void)
     gtk_widget_set_sensitive (langmenu, FALSE);
 
     /* If sysadmin enabled shutdown menu */
-    if (gdmShutdownMenu) {
+    if (GdmShutdownMenu) {
 	submenu = gtk_menu_new();
 
 	item = gtk_menu_item_new_with_label (_("Reboot..."));
@@ -987,7 +990,7 @@ gdm_greeter_buttons_init(void)
 			   GTK_SIGNAL_FUNC (gdm_greeter_halt_handler), NULL);
 	gtk_widget_show(item);
 
-	if(gdmSuspend) {
+	if(GdmSuspend) {
 	    item = gtk_menu_item_new_with_label (_("Suspend"));
 	    gtk_menu_append(GTK_MENU (submenu), item);
 	    gtk_signal_connect(GTK_OBJECT (item), "activate",
@@ -1001,10 +1004,10 @@ gdm_greeter_buttons_init(void)
 	gtk_widget_show(item);
     }
 
-    if(gdmIconify) {
-	if(access(gdmIconFile, R_OK)) {
-	    syslog(LOG_WARNING, _("Can't open icon file: %s. Suspending iconify feature!"), gdmIconFile);
-	    gdmIconify=0;
+    if(GdmIconify) {
+	if(access(GdmIconFile, R_OK)) {
+	    syslog(LOG_WARNING, _("Can't open icon file: %s. Suspending iconify feature!"), GdmIconFile);
+	    GdmIconify=0;
 	}
 	else {
 	    item = gtk_menu_item_new_with_label (_("Iconify"));
@@ -1109,7 +1112,7 @@ gdm_greeter_browser_init (void)
     style->bg[GTK_STATE_NORMAL] = bbg;
     gtk_widget_push_style(style);
 
-    browser = GNOME_ICON_LIST (gnome_icon_list_new (100, NULL, FALSE));
+    browser = GNOME_ICON_LIST (gnome_icon_list_new (GdmIconMaxWidth+20, NULL, FALSE));
     gnome_icon_list_freeze (GNOME_ICON_LIST (browser));
     gtk_widget_pop_style();
 
@@ -1192,8 +1195,8 @@ gdm_greeter_login_entry_handler (GtkWidget *widget, GdkEventKey *event)
 	 */
 	if(lmatch)
 	    gtk_entry_set_text(GTK_ENTRY (loginentry), lmatch);
-
-	if(gdmDisplayBrowser) {
+	
+	if(GdmDisplayBrowser) {
 	    gnome_icon_list_freeze(browser);
 	    gnome_icon_list_clear(browser);
 	    g_list_foreach(result, (GFunc) gdm_greeter_browser_foreach, NULL);
@@ -1204,12 +1207,12 @@ gdm_greeter_login_entry_handler (GtkWidget *widget, GdkEventKey *event)
 	    user=g_list_nth_data(result, i);
 	    
 	    if(!strcasecmp(gtk_entry_get_text(GTK_ENTRY(loginentry)), user->login)) {
-
-		if(gdmDisplayBrowser) {
+		
+		if(GdmDisplayBrowser) {
 		    gnome_icon_list_moveto(browser, i, 0.5);
 		    gnome_icon_list_select_icon(browser, i);
 		}
-
+		
 		gdm_greeter_sl_update(user);
 	    }
 	}
@@ -1353,14 +1356,14 @@ gdm_greeter_user_alloc (gchar *logname, gint uid, gchar *homedir)
     user->homedir=g_strdup(homedir);
 
     /* If browser enabled choose an appropriate picture */
-    if(gdmDisplayBrowser) {
+    if(GdmDisplayBrowser) {
 	
 	/* Check if ~user/.gnome/photo passes sanity check */
 	dir=g_strconcat(homedir, "/.gnome", NULL);
 	fileok=gdm_file_check("gdm_greeter_user_alloc", uid, dir, "photo", FALSE);
 	g_free(dir);
 
-	globimg=g_strconcat(gdmGlobalImageDir, "/", logname, NULL);
+	globimg=g_strconcat(GdmGlobalImageDir, "/", logname, NULL);
 
 	/* If user's picture passed sanity check/size cutoff value load it */
 	if(fileok) 
@@ -1376,13 +1379,13 @@ gdm_greeter_user_alloc (gchar *logname, gint uid, gchar *homedir)
 	    w=imlibimg->rgb_width;
 	    h=imlibimg->rgb_height;
 	    
-	    if(w>h && w>gdmIconMaxWidth) {
-		w=gdmIconMaxWidth;
-		h=h*((gfloat) gdmIconMaxWidth/w);
+	    if(w>h && w>GdmIconMaxWidth) {
+		w=GdmIconMaxWidth;
+		h=h*((gfloat) GdmIconMaxWidth/w);
 	    } 
-	    else if(h>gdmIconMaxHeight) {
-		w=w*((gfloat) gdmIconMaxHeight/h);
-		h=gdmIconMaxHeight;
+	    else if(h>GdmIconMaxHeight) {
+		w=w*((gfloat) GdmIconMaxHeight/h);
+		h=GdmIconMaxHeight;
 	    }
 
 	    user->picture=gdk_imlib_clone_scaled_image(imlibimg, w, h);
@@ -1390,7 +1393,7 @@ gdm_greeter_user_alloc (gchar *logname, gint uid, gchar *homedir)
 	    gdk_imlib_destroy_image(imlibimg);
 	}
 	else
-	    user->picture=gdmNofaceImage;
+	    user->picture=nofaceimg;
 
 	g_free(globimg);
 	g_free(img);
@@ -1442,20 +1445,20 @@ gdm_greeter_users_init (void)
 {
     struct passwd *pwent;
 
-    if(gdmDisplayBrowser) {
-	if(access(gdmNofaceImageFile, R_OK)) {
-	    syslog(LOG_WARNING, _("Can't open NofaceImageFile: %s. Suspending face browser!"), gdmNofaceImageFile);
-	    gdmDisplayBrowser=0;
+    if(GdmDisplayBrowser) {
+	if(access(GdmNofaceImageFile, R_OK)) {
+	    syslog(LOG_WARNING, _("Can't open NofaceImageFile: %s. Suspending face browser!"), GdmNofaceImageFile);
+	    GdmDisplayBrowser=0;
 	}
 	else 
-	    gdmNofaceImage=gdk_imlib_load_image(gdmNofaceImageFile);
+	    nofaceimg=gdk_imlib_load_image(GdmNofaceImageFile);
     }
 
     pwent = getpwent();
 	
     while (pwent != NULL) {
 	
-	if (gdm_greeter_check_shell(pwent->pw_shell)) {
+	if (pwent->pw_shell && gdm_greeter_check_shell(pwent->pw_shell)) {
 	    users=g_list_insert_sorted(users, 
 				       gdm_greeter_user_alloc(g_strdup(pwent->pw_name),
 							      pwent->pw_uid,
@@ -1489,10 +1492,25 @@ main (int argc, char *argv[])
     openlog("gdmgreeter", LOG_PID, LOG_DAEMON);
 
     gnome_init("gdmgreeter", VERSION, argc, argv);
-
+    gnome_sound_shutdown ();
+    
     gdm_greeter_parse_config();
 
-    gdm_greeter_users_init();
+    /* If completion is disabled => Disable browser */
+    /* if(!GdmCompletion)
+	GdmDisplayBrowser=FALSE; */
+
+    /* If browser is enabled => Force completion */
+    /* if(GdmDisplayBrowser) */
+
+    /* Force completion until PAM conversation function is implemented */
+    GdmCompletion=TRUE;
+
+    if(GdmCompletion)
+	gdm_greeter_users_init();
+
+    if(GdmGtkRC)
+	gtk_rc_parse(GdmGtkRC);
 
     greeter = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 
@@ -1507,7 +1525,7 @@ main (int argc, char *argv[])
     gtk_container_border_width (GTK_CONTAINER (gdmMain), 0);
     gtk_container_add(GTK_CONTAINER (greeterframe), gdmMain);
 
-    if(gdmDisplayBrowser) {
+    if(GdmDisplayBrowser) {
 	gdm_greeter_browser_init();
 	gtk_box_pack_start(GTK_BOX (gdmMain), 
 			   GTK_WIDGET(browser_hbox), TRUE, TRUE, 0);
@@ -1518,7 +1536,7 @@ main (int argc, char *argv[])
     gtk_box_pack_end(GTK_BOX (gdmMain), 
                        gdmLogoCtrlHbox, FALSE, FALSE, 0);
 
-    if ((gdmDisplayLogo) && (gdmLogoFilename)) {
+    if ((GdmDisplayLogo) && (GdmLogoFilename)) {
 	logobox = gtk_drawing_area_new();
 	gdm_greeter_logo_init();
 	gtk_box_pack_start(GTK_BOX (gdmLogoCtrlHbox), 
@@ -1552,7 +1570,7 @@ main (int argc, char *argv[])
 
     rootwin=gdk_window_foreign_new (GDK_ROOT_WINDOW ());
 
-    if(gdmDisplayBrowser) {
+    if(GdmDisplayBrowser) {
 	gtk_widget_show_all(GTK_WIDGET(browser));
 	
 	gtk_widget_set_usize(GTK_WIDGET (greeter), 
