@@ -28,24 +28,27 @@
 
 #include "gdm.h"
 
-static const gchar RCSid[]="$Id: gdmmisc.c,v 1.12 1999/01/27 22:11:27 mkp Exp $";
+static const gchar RCSid[]="$Id: gdmmisc.c,v 1.15 1999/02/08 22:43:35 mkp Exp $";
 
 
 gchar **gdm_arg_munch(gchar *p);
 gboolean gdm_file_check(gchar *caller, uid_t user, gchar *dir, gchar *file, gboolean absentok);
 gchar *gdm_parse_enriched_string(gchar *s);
-gint gdm_exec_script(gdmDisplayType *d, gchar *dir);
+gint gdm_exec_script(GdmDisplay *d, gchar *dir);
 void gdm_exec_command(gchar *cmd);
+void gdm_fail(const gchar *format, ...);
 void gdm_abort(const gchar *format, ...);
-void gdm_return(const gchar *format, ...);
+void gdm_info(const gchar *format, ...);
 void gdm_error(const gchar *format, ...);
 void gdm_debug(const gchar *format, ...);
 void gdm_remanage(const gchar *format, ...);
 void gdm_putenv(gchar *s);
 
 
-extern gchar *gdmPidFile;
-extern gint gdmUserMaxFile;
+extern gchar *GdmPidFile;
+extern gint GdmUserMaxFile;
+extern gchar *GdmDefaultPath;
+extern gint GdmDebug;
 
 gchar **
 gdm_arg_munch(gchar *p)
@@ -152,7 +155,7 @@ gdm_file_check(gchar *caller, uid_t user, gchar *dir, gchar *file, gboolean abse
     };
 
     /* ... and smaller than sysadmin specified limit. */
-    if(statbuf.st_size > gdmUserMaxFile) {
+    if(statbuf.st_size > GdmUserMaxFile) {
 	syslog(LOG_WARNING, _("%s: %s is bigger than sysadmin specified maximum file size. Skipping."), caller, str);
 	g_free(str);
 	return(FALSE);
@@ -171,9 +174,24 @@ gdm_parse_enriched_string(gchar *s)
     gchar cmd;
     gchar *buffer;
     gchar *start;
-    gchar *hostname;
+    gchar hostbuf[256];
+    gchar *hostname, *temp1, *temp2, *display;
 
-    hostname=getenv("HOSTNAME");
+    display=getenv("DISPLAY");
+
+    if(!display)
+	return(NULL);
+
+    temp1 = strchr(display, '.');
+    temp2 = strchr(display, ':');
+
+    if(temp1)
+	*temp1='\0';
+    else 
+	*temp2='\0';
+
+    gethostname(hostbuf, 255);
+    hostname=g_strdup(hostbuf);
 
     if(!hostname) 
 	hostname=g_strdup("Gnome");
@@ -181,12 +199,12 @@ gdm_parse_enriched_string(gchar *s)
     if(strlen(s) > 1023) {
 	syslog(LOG_ERR, _("gdm_parse_enriched_string: String too long!"));
 	return(g_strconcat(_("Welcome to "), hostname, NULL));
-    };
+    }
 
     if(!(buffer = g_malloc(4096))) {
 	syslog(LOG_ERR, _("gdm_parse_enriched_string: Could not malloc temporary buffer!"));
 	return(NULL);
-    };
+    }
 
     start = buffer;
 
@@ -202,6 +220,11 @@ gdm_parse_enriched_string(gchar *s)
 		buffer+=strlen(hostname);
 		break;
 		
+	    case 'd': 
+		memcpy(buffer, display, strlen(display));
+		buffer+=strlen(display);
+		break;
+
 	    case '%':
 		*buffer++='%';
 		break;
@@ -222,7 +245,7 @@ gdm_parse_enriched_string(gchar *s)
 
 /* Execute a script and wait for it to finish */
 gint 
-gdm_exec_script(gdmDisplayType *d, gchar *dir)
+gdm_exec_script(GdmDisplay *d, gchar *dir)
 {
     pid_t pid;
     gchar *script, *defscript, *scr;
@@ -242,6 +265,7 @@ gdm_exec_script(gdmDisplayType *d, gchar *dir)
     switch(pid=fork()) {
 	    
     case 0:
+	gdm_putenv(g_strconcat("PATH=", GdmDefaultPath, NULL));
 	argv=gdm_arg_munch(scr);
 	execv(argv[0], argv);
 	syslog(LOG_ERR, _("gdm_exec_script: Failed starting: %s"), scr);
@@ -273,6 +297,7 @@ gdm_exec_command(gchar *cmd)
     switch(pid=fork()) {
 	    
     case 0:
+	gdm_putenv(g_strconcat("PATH=", GdmDefaultPath, NULL));
 	argv=gdm_arg_munch(cmd);
 	execvp(argv[0], argv);
 	syslog(LOG_ERR, _("gdm_exec_command: Failed starting: %s"), cmd);
@@ -290,7 +315,7 @@ gdm_exec_command(gchar *cmd)
 
 /* Log error and abort master daemon */
 void 
-gdm_abort(const gchar *format, ...)
+gdm_fail(const gchar *format, ...)
 {
     va_list args;
     gchar *s;
@@ -305,38 +330,16 @@ gdm_abort(const gchar *format, ...)
     fflush(stderr);
 
     g_free(s);
-    unlink(gdmPidFile);
+    unlink(GdmPidFile);
     closelog();
 
     exit(EXIT_FAILURE);
 }
 
 
-/* Like gdm_abort but return instead of exit */
-void 
-gdm_return(const gchar *format, ...)
-{
-    va_list args;
-    gchar *s;
-
-    va_start(args, format);
-    s=g_strdup_vprintf(format, args);
-    va_end(args);
-
-    syslog(LOG_ERR, s);
-    fprintf(stderr, s);
-    fprintf(stderr, "\n");
-    fflush(stderr);
-
-    g_free(s);
-    unlink(gdmPidFile);
-    closelog();
-}
-
-
 /* Log error and abort the slave daemon */
 void 
-gdm_error(const gchar *format, ...)
+gdm_abort(const gchar *format, ...)
 {
     va_list args;
     gchar *s;
@@ -353,7 +356,7 @@ gdm_error(const gchar *format, ...)
 }
 
 
-/* Log non fatal error and restart display */
+/* Remanage display */
 void 
 gdm_remanage(const gchar *format, ...)
 {
@@ -372,6 +375,40 @@ gdm_remanage(const gchar *format, ...)
 }
 
 
+/* Log non fatal error/message */
+void 
+gdm_info(const gchar *format, ...)
+{
+    va_list args;
+    gchar *s;
+
+    va_start(args, format);
+    s=g_strdup_vprintf(format, args);
+    va_end(args);
+    
+    syslog(LOG_INFO, s);
+    
+    g_free(s);
+}
+
+
+/* Log error condition */
+void 
+gdm_error(const gchar *format, ...)
+{
+    va_list args;
+    gchar *s;
+
+    va_start(args, format);
+    s=g_strdup_vprintf(format, args);
+    va_end(args);
+    
+    syslog(LOG_ERR, s);
+    
+    g_free(s);
+}
+
+
 /* Log debug messages */
 void 
 gdm_debug(const gchar *format, ...)
@@ -379,7 +416,7 @@ gdm_debug(const gchar *format, ...)
     va_list args;
     gchar *s;
 
-    if(DEBUG) {
+    if(GdmDebug) {
 	va_start(args, format);
 	s=g_strdup_vprintf(format, args);
 	va_end(args);
