@@ -34,30 +34,31 @@
 
 #include "gdm.h"
 
-static const gchar RCSid[]="$Id: gdmserver.c,v 1.3 1999/01/27 22:11:28 mkp Exp $";
+static const gchar RCSid[]="$Id: gdmserver.c,v 1.6 1999/02/24 22:25:47 mkp Exp $";
 
-extern gchar *gdmDisplayInit;
-extern gchar *gdmAuthDir;
-extern gchar *gdmLogDir;
+extern gchar *GdmDisplayInit;
+extern gchar *GdmAuthDir;
+extern gchar *GdmLogDir;
 
 extern gchar **gdm_arg_munch(const gchar *p);
-extern void gdm_auth_secure_display(gdmDisplayType *);
+extern void gdm_auth_secure_display(GdmDisplay *);
 extern void gdm_debug(const gchar *, ...);
-extern void gdm_abort(const gchar *, ...);
-extern void gdm_display_manage(gdmDisplayType *);
+extern void gdm_error(const gchar *, ...);
+extern gint gdm_display_manage(GdmDisplay *);
 extern void gdm_putenv(gchar *);
 
-void gdm_server_start(gdmDisplayType *d);
-void gdm_server_stop(gdmDisplayType *d);
-void gdm_server_restart(gdmDisplayType *d);
+void gdm_server_start(GdmDisplay *d);
+void gdm_server_stop(GdmDisplay *d);
+void gdm_server_restart(GdmDisplay *d);
 void gdm_server_usr1_handler(gint);
+GdmDisplay *gdm_server_alloc (gint id, gchar *command);
 
-gdmDisplayType *d;
-Display *dsp=NULL;
+
+GdmDisplay *d;
 
 
 void 
-gdm_server_start(gdmDisplayType *disp)
+gdm_server_start(GdmDisplay *disp)
 {    
     struct sigaction usr1;
     sigset_t usr1mask;
@@ -71,20 +72,20 @@ gdm_server_start(gdmDisplayType *disp)
 
     /* Catch USR1 from X server */
     usr1.sa_handler = gdm_server_usr1_handler;
-    usr1.sa_flags = SA_RESTART|SA_ONESHOT;
+    usr1.sa_flags = SA_RESTART|SA_RESETHAND;
     sigemptyset(&usr1.sa_mask);
 
     if(sigaction(SIGUSR1, &usr1, NULL) < 0) {
         syslog(LOG_ERR, _("gdm_server_start: Error setting up USR1 signal handler"));
 	exit(SERVER_ABORT);
-    };
+    }
 
     sigemptyset(&usr1mask);
     sigaddset(&usr1mask, SIGUSR1);
     sigprocmask(SIG_UNBLOCK, &usr1mask, NULL);
 
     /* Log all output from spawned programs to a file */
-    logfd=open(g_strconcat(gdmLogDir, "/", d->name, ".log", NULL),
+    logfd=open(g_strconcat(GdmLogDir, "/", d->name, ".log", NULL),
 	       O_CREAT|O_TRUNC|O_APPEND|O_WRONLY, 0666);
 
     if(logfd != -1) {
@@ -98,7 +99,7 @@ gdm_server_start(gdmDisplayType *disp)
     if(d->servpid) {
 	gdm_debug(_("gdm_server_start: Old server found (%d). Killing."), d->servpid);
 	gdm_server_stop(d);
-    };
+    }
 
     /* Secure display with cookie */
     gdm_auth_secure_display(d);
@@ -120,9 +121,9 @@ gdm_server_start(gdmDisplayType *disp)
 	if(sigaction(SIGUSR1, &usr1, NULL) < 0) {
 	    syslog(LOG_ERR, _("gdm_server_start: Error setting USR1 to SIG_IGN"));
 	    exit(SERVER_ABORT);
-	};
+	}
 	
-	srvcmd=g_strconcat(d->command, " -auth ", gdmAuthDir, \
+	srvcmd=g_strconcat(d->command, " -auth ", GdmAuthDir, \
 			      "/", d->name, ".xauth ", 
 			      d->name, NULL);
 
@@ -145,17 +146,17 @@ gdm_server_start(gdmDisplayType *disp)
 	
     default:
 	break;
-    };
+    }
 
     d->servstat=SERVER_STARTED;
 
     /* Wait for X server to send ready signal */
     pause();
-};
+}
 
 
 void
-gdm_server_stop(gdmDisplayType *d)
+gdm_server_stop(GdmDisplay *d)
 {
     gdm_debug(_("gdm_server_stop: Server for %s going down!"), d->name);
 
@@ -170,13 +171,19 @@ gdm_server_stop(gdmDisplayType *d)
 
 
 void
-gdm_server_restart(gdmDisplayType *d)
+gdm_server_restart(GdmDisplay *d)
 {
     sigset_t usr1mask;
     struct sigaction usr1;
 
     gdm_debug(_("gdm_server_restart: Server for %s restarting!"), d->name);
 
+    if(d->servpid && !kill(d->servpid, 0)) {
+	gdm_debug(_("gdm_server_restart: Old server for %s still alive. Killing!"), d->name);
+	gdm_server_stop(d);
+	gdm_server_start(d);
+    }
+
     /* Create new cookie */
     gdm_auth_secure_display(d);
     gdm_putenv(g_strconcat("XAUTHORITY=", d->auth, NULL));
@@ -184,22 +191,19 @@ gdm_server_restart(gdmDisplayType *d)
 
     /* Catch USR1 from X server */
     usr1.sa_handler = gdm_server_usr1_handler;
-    usr1.sa_flags = SA_RESTART|SA_ONESHOT;
+    usr1.sa_flags = SA_RESTART|SA_RESETHAND;
     sigemptyset(&usr1.sa_mask);
 
     if(sigaction(SIGUSR1, &usr1, NULL) < 0) {
         syslog(LOG_ERR, _("gdm_server_start: Error setting up USR1 signal handler"));
 	exit(SERVER_ABORT);
-    };
+    }
 
     sigemptyset(&usr1mask);
     sigaddset(&usr1mask, SIGUSR1);
     sigprocmask(SIG_UNBLOCK, &usr1mask, NULL);
 
-    /* Prepare to rise again */
-    d->servstat=SERVER_PHOENIX;
-
-    /* Reset X and force auth file reread. XCloseDisplay works on most server, but we play it safe */
+    /* Reset X and force auth file reread. XCloseDisplay works on most servers but we play it safe */
     kill(d->servpid, SIGHUP);
 
     d->servstat=SERVER_STARTED;
@@ -228,23 +232,51 @@ gdm_server_usr1_handler(gint sig)
      * X server resetting due to lack of active connections. */
 
     gdm_debug(_("gdm_slave_start: Opening display %s"), d->name);
-    dsp=NULL;
+    d->dsp=NULL;
 
-    while(openretries < 10 && dsp==NULL) {
-	dsp=XOpenDisplay(d->name);
+    while(openretries < 10 && d->dsp==NULL) {
+	d->dsp=XOpenDisplay(d->name);
 
-	if(!dsp) {
+	if(!d->dsp) {
 	    gdm_debug(_("gdm_server_usr1_handler: Sleeping %d on a retry"), openretries*2);
 	    sleep(openretries*2);
 	    openretries++;
-	};
-    };
+	}
+    }
     
-    if(!dsp) 
-	gdm_abort(_("gdm_server_usr1_handler: Could not open display %s"), d->name);
+    if(d->dsp) 
+	gdm_display_manage(d);
+    else
+	gdm_error(_("gdm_server_usr1_handler: Could not open display %s"), d->name);
+}
 
-    /* Start greeter on display */
-    gdm_display_manage(d);
+
+GdmDisplay * 
+gdm_server_alloc (gint id, gchar *command)
+{
+    gchar dname[1024];
+
+    GdmDisplay *d = g_malloc(sizeof(GdmDisplay));
+
+    sprintf(dname, ":%d", id);    
+    d->auth = NULL;
+    d->command = g_strdup(command);
+    d->cookie = NULL;
+    d->dispstat = DISPLAY_DEAD;
+    d->greetpid = 0;
+    d->id = id;
+    d->name = strdup(dname);
+    d->servpid = 0;
+    d->servstat = SERVER_DEAD;
+    d->sessionid = 0;
+    d->sesspid = 0;
+    d->slavepid = 0;
+    d->type = DISPLAY_LOCAL;
+    d->sessionid = 0;
+    d->acctime = 0;
+    d->dsp = NULL;
+
+    return (d);
 }
 
 		
