@@ -46,33 +46,34 @@ pam_handle_t *pamh;
 gint pamh=0;
 #endif /* HAVE_PAM */
 
-static const gchar RCSid[]="$Id: gdmslave.c,v 1.35 1999/01/27 22:11:29 mkp Exp $";
+static const gchar RCSid[]="$Id: gdmslave.c,v 1.41 1999/02/24 22:25:48 mkp Exp $";
 
-extern uid_t gdmUserId;
-extern gid_t gdmGroupId;
-extern gchar *gdmSessDir;
-extern gchar *gdmGreeter;
-extern gchar *gdmDisplayInit;
-extern gchar *gdmPreSession;
-extern gchar *gdmPostSession;
-extern gchar *gdmSuspend;
-extern gchar *gdmDefaultPath;
-extern gint  gdmKillInitClients;
+extern uid_t GdmUserId;
+extern gid_t GdmGroupId;
+extern gchar *GdmSessDir;
+extern gchar *GdmGreeter;
+extern gchar *GdmDisplayInit;
+extern gchar *GdmPreSession;
+extern gchar *GdmPostSession;
+extern gchar *GdmSuspend;
+extern gchar *GdmDefaultPath;
+extern gint  GdmKillInitClients;
 
 extern gboolean gdm_file_check(gchar *caller, uid_t user, gchar *dir, gchar *file, gboolean absentok);
 extern gchar **gdm_arg_munch(const gchar *p);
-extern gint gdm_exec_script(gdmDisplayType*, gchar *dir);
+extern gint gdm_exec_script(GdmDisplay*, gchar *dir);
 extern void gdm_abort(const char*, ...);
+extern void gdm_error(const char*, ...);
 extern void gdm_debug(const char*, ...);
 extern void gdm_remanage(const char*, ...);
 extern gint gdm_verify_user (gchar *login, gchar *passwd, gchar *display);
 extern void gdm_verify_cleanup (void);
-extern void gdm_auth_user_add(gdmDisplayType *d, gchar *home);
-extern void gdm_auth_user_remove(gdmDisplayType *d, gchar *home);
+extern void gdm_auth_user_add(GdmDisplay *d, gchar *home);
+extern void gdm_auth_user_remove(GdmDisplay *d, gchar *home);
 extern void gdm_exec_command(gchar *cmd);
 extern void gdm_putenv(gchar *s);
 
-void        gdm_slave_start(gdmDisplayType *d);
+void        gdm_slave_start(GdmDisplay *d);
 static gint gdm_slave_xerror_handler(Display *disp, XErrorEvent *evt);
 static gint gdm_slave_xioerror_handler(Display *disp);
 static void gdm_slave_greeter(void);
@@ -83,19 +84,24 @@ static void gdm_slave_session_cleanup(void);
 static void gdm_slave_term_handler(int sig);
 static void gdm_slave_child_handler(int sig);
 static void gdm_slave_windows_kill(void);
+static void gdm_slave_xsync_handler(int sig);
+static gboolean gdm_slave_xsync_ping(void);
 
-gdmDisplayType *d;
+
+GdmDisplay *d;
 struct passwd *pwent;
 sigset_t mask, omask;
 FILE *greeter;
+gboolean pingack;
+gboolean greet=TRUE;
 
 
 void 
-gdm_slave_start(gdmDisplayType *display)
+gdm_slave_start(GdmDisplay *display)
 {    
     struct sigaction term, child;
 
-    gdm_debug(_("gdm_slave_start: Starting slave process for %s"), d->name);
+    gdm_debug(_("gdm_slave_start: Starting slave process for %s"), display->name);
 
     d=display;
 
@@ -133,24 +139,29 @@ gdm_slave_start(gdmDisplayType *display)
     gdm_slave_greeter();
 }
 
+#define FIELD_SIZE 64
 
 static void
 gdm_slave_greeter(void)
 {
     gint pipeline[2];
     gchar cmd;
-    gchar login[64];
-    gchar passwd[64];
-    gchar buf[65];
-
+    gchar login[FIELD_SIZE];
+    gchar passwd[FIELD_SIZE];
+    gchar buf[FIELD_SIZE+1];
+    gchar **argv;
+    int len;
+    
     gdm_debug(_("gdm_slave_greeter: Running greeter on %s"), d->name);
 
     /* Run the init script. gdmslave suspends until script has terminated */
-    gdm_exec_script(d, gdmDisplayInit);
+    gdm_exec_script(d, GdmDisplayInit);
 
     /* Open a pipe for greeter communications */
     if(pipe(pipeline) < 0) 
 	gdm_abort(_("gdm_slave_greeter: Can't init pipe to gdmgreeter"));
+
+    greet=TRUE;
     
     /* Fork. Parent is gdmslave, child is greeter process. */
     switch(d->greetpid=fork()) {
@@ -169,17 +180,19 @@ gdm_slave_greeter(void)
 	close(STDOUT_FILENO);
 	dup2(pipeline[1], STDOUT_FILENO);
 
-        if(setgid(gdmGroupId) < 0) 
-	    gdm_abort(_("gdm_slave_greeter: Couldn't set groupid to %d"), gdmGroupId);
+        if(setgid(GdmGroupId) < 0) 
+	    gdm_abort(_("gdm_slave_greeter: Couldn't set groupid to %d"), GdmGroupId);
 
-        if(setuid(gdmUserId) < 0) 
-	    gdm_abort(_("gdm_slave_greeter: Couldn't set userid to %d"), gdmUserId);
+        if(setuid(GdmUserId) < 0) 
+	    gdm_abort(_("gdm_slave_greeter: Couldn't set userid to %d"), GdmUserId);
 
 	gdm_putenv(g_strconcat("XAUTHORITY=", d->auth, NULL));
 	gdm_putenv(g_strconcat("DISPLAY=", d->name, NULL));
 	gdm_putenv(g_strconcat("HOME=/", NULL));
+	gdm_putenv(g_strconcat("PATH=", GdmDefaultPath, NULL));
 
-	execl(gdmGreeter, gdmGreeter, NULL);
+	argv=gdm_arg_munch(GdmGreeter);
+	execv(argv[0], argv);
 
 	gdm_abort(_("gdm_slave_greeter: Error starting greeter on display %s"), d->name);
 	
@@ -189,7 +202,7 @@ gdm_slave_greeter(void)
     default:
 	gdm_debug(_("gdm_slave_greeter: Greeter on pid %d"), d->greetpid);
 	break;
-    };
+    }
 
     greeter = fdopen(pipeline[0], "r");
     close(pipeline[1]);
@@ -213,19 +226,21 @@ gdm_slave_greeter(void)
 		exit(DISPLAY_REBOOT);
 
 	    case 'S':		/* Suspend for laptops */
-		gdm_exec_command(gdmSuspend);
+		gdm_exec_command(GdmSuspend);
 		break;
 
 	    case 'U':		/* User session initiated */
-		fgets(buf, 64, greeter);
+		fgets(buf, FIELD_SIZE, greeter);
 		if (sscanf(buf, "%s", login) != 1)
 		    login[0] = 0;
 		
-                fgets(buf, 64, greeter);
-                if (sscanf(buf, "%s", passwd) != 1)
-		    passwd[0] = 0;
+                fgets(buf, FIELD_SIZE, greeter);
+		len = strlen (buf);
+		if (buf [strlen (buf)-1] == '\n')
+			buf [strlen (buf)-1] = 0;
+		strncpy (passwd, buf, FIELD_SIZE);
  	
-		if(strlen(login)+strlen(passwd)==0) 
+		if(strlen(login)+strlen(passwd)==0)
 		    gdm_remanage(_("gdm_slave_greeter: Insufficient data from greeter in pipe"));
 
 		/* Verify login/passwd pair */
@@ -244,34 +259,38 @@ gdm_slave_greeter(void)
 static void
 gdm_slave_get_opts(gchar *login)
 {
-    gchar lang[64];
+    gchar lang[FIELD_SIZE];
     gchar session[_POSIX_PATH_MAX+1];
     gchar buf[_POSIX_PATH_MAX+1];
     gint savesess=0, savelang=0;
     gchar cmd;
 
+    lang [0] = 0;
+    session [0] = 0;
+    buf [0] = 0;
+    
     kill(d->greetpid, SIGUSR2);	/* Request session and lang info */
     
     while((cmd=fgetc(greeter)) != EOF) {
 
 	if(cmd==0x3) {
-	    fgets(buf, _POSIX_PATH_MAX, greeter);
-	    if(sscanf(buf, "%s", session) != 1)
-		session[0]=0;
+	    fgets(session, sizeof (session)-1, greeter);
+	    if (session [strlen (session)-1] == '\n')
+		    session [strlen (session)-1] = 0;
 	    
-	    fgets(buf, 64, greeter);
+	    fgets(buf, FIELD_SIZE, greeter);
 	    if(sscanf(buf, "%d", &savesess) != 1)
 		savesess=0;
     
-	    fgets(buf, 64, greeter);
-	    if(sscanf(buf, "%s", lang) != 1)
-		lang[0]=0;
+	    fgets(lang, sizeof (lang)-1, greeter);
+	    if (lang [strlen (lang)-1] == '\n')
+		    lang [strlen (lang)-1] = 0;
 
-	    fgets(buf, 64, greeter);
+	    fgets(buf, sizeof (buf)-1, greeter);
 	    if(sscanf(buf, "%d", &savelang) != 1)
 		savelang=0;
     
-	    if(strlen(session)+strlen(lang)==0) 
+	    if(strlen(session)+strlen(lang)==0)
 		gdm_remanage(_("gdm_slave_get_opts: Insufficient data from greeter in pipe"));
 
 	    sigemptyset(&mask);
@@ -279,6 +298,8 @@ gdm_slave_get_opts(gchar *login)
 	    sigprocmask(SIG_BLOCK, &mask, &omask);
 	    
 	    /* Kill greeter and wait for it to die */
+	    greet=FALSE;
+
 	    kill(d->greetpid, SIGINT);
 	    waitpid(d->greetpid, 0, 0); 
 	    d->greetpid=0;
@@ -286,7 +307,7 @@ gdm_slave_get_opts(gchar *login)
 	    sigprocmask(SIG_UNBLOCK, &omask, NULL);
 	    
 	    /* Kill clients started by Init script */
-	    if(gdmKillInitClients)
+	    if(GdmKillInitClients)
 		gdm_slave_windows_kill();
 
 	    /* Run the user session */
@@ -318,7 +339,7 @@ gdm_slave_session_start(gchar *login, gchar *session, gint savesess, gchar *lang
     gdm_putenv(g_strconcat("HOME=", pwent->pw_dir, NULL));
     gdm_putenv(g_strconcat("GDMSESSION=", session, NULL));
     gdm_putenv(g_strconcat("SHELL=", pwent->pw_shell, NULL));
-    gdm_putenv(g_strconcat("PATH=", gdmDefaultPath, NULL));
+    gdm_putenv(g_strconcat("PATH=", GdmDefaultPath, NULL));
 
     if(!strcasecmp(lang, "english"))
 	putenv("LANG=C");
@@ -326,7 +347,7 @@ gdm_slave_session_start(gchar *login, gchar *session, gint savesess, gchar *lang
 	gdm_putenv(g_strconcat("LANG=", lang, NULL));
 
     /* If script fails reset X server and restart greeter */
-    if(gdm_exec_script(d, gdmPreSession) != EXIT_SUCCESS) 
+    if(gdm_exec_script(d, GdmPreSession) != EXIT_SUCCESS) 
 	gdm_remanage(_("gdm_slave_session_init: Execution of PreSession script returned > 0. Aborting."));
 
     switch(d->sesspid=fork()) {
@@ -360,11 +381,13 @@ gdm_slave_session_start(gchar *login, gchar *session, gint savesess, gchar *lang
 	if(stat(cfgdir, &statbuf) == -1) { /* FIXME: Maybe I need to be a bit paranoid here! */
 	    mkdir(cfgdir, S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
 	    chmod(cfgdir, S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
-	};
+	}
 	    
 	/* Sanity check on ~user/.gnome/gdm */
-	if(!gdm_file_check("gdm_slave_session_init", pwent->pw_uid, cfgdir, "gdm", TRUE))
+	if(!gdm_file_check("gdm_slave_session_init", pwent->pw_uid, cfgdir, "gdm", TRUE)) {
+	    gdm_slave_session_cleanup();
 	    exit(DISPLAY_REMANAGE);
+	}
 
 	g_free(cfgdir);
 
@@ -374,34 +397,37 @@ gdm_slave_session_start(gchar *login, gchar *session, gint savesess, gchar *lang
 	    gnome_config_set_string(cfgstr, session);
 	    gnome_config_sync();
 	    g_free(cfgstr);
-	};
+	}
 
 	if(savelang) {
 	    cfgstr = g_strconcat("=", pwent->pw_dir, "/.gnome/gdm=/session/lang", NULL);
 	    gnome_config_set_string(cfgstr, lang);
 	    gnome_config_sync();
 	    g_free(cfgstr);
-	};
+	}
 
-	sessdir = g_strconcat(gdmSessDir, "/", session, NULL);
+	sessdir = g_strconcat(GdmSessDir, "/", session, NULL);
 
 	gdm_debug("Running %s for %s on %s", sessdir, login, d->name);
 
 	execl(sessdir, NULL);
 	
 	syslog(LOG_ERR, _("gdm_slave_session_init: Could not start session `%s'"), sessdir);
+
 	gdm_slave_session_stop();
+	gdm_slave_session_cleanup();
+
 	exit(DISPLAY_REMANAGE);
 	
     default:
 	break;
-    };
+    }
 
     waitpid(d->sesspid, 0, 0);
 
     gdm_slave_session_stop();
     gdm_slave_session_cleanup();
-};
+}
 
 
 static void
@@ -419,9 +445,9 @@ gdm_slave_session_stop(void)
     if(gdm_file_check("gdm_slave_session_stop", pwent->pw_uid, pwent->pw_dir, ".Xauthority", FALSE)) 
 	gdm_auth_user_remove(d, pwent->pw_dir);
 
-    setegid(gdmGroupId);
+    setegid(GdmGroupId);
     seteuid(0);
-};
+}
 
 
 /* Only executed if display is still alive */
@@ -430,19 +456,18 @@ gdm_slave_session_cleanup(void)
 {
     gdm_debug(_("gdm_slave_session_cleanup: %s on %s"), pwent->pw_name, d->name);
 
-    /* X error handlers to avoid the default ones: exit(1) */
-    XSetErrorHandler(gdm_slave_xerror_handler);
-    XSetIOErrorHandler(gdm_slave_xioerror_handler);
+    if(d->dsp && gdm_slave_xsync_ping()) {
 
-    /* Execute post session script */
-    gdm_debug(_("gdm_slave_session_cleanup: Running post session script"));
-    gdm_exec_script(d, gdmPostSession);
+	/* Execute post session script */
+	gdm_debug(_("gdm_slave_session_cleanup: Running post session script"));
+	gdm_exec_script(d, GdmPostSession);
 
-    /* Cleanup */
-    gdm_debug(_("gdm_slave_session_cleanup: Killing windows"));
-    gdm_slave_windows_kill();
+	/* Cleanup */
+	gdm_debug(_("gdm_slave_session_cleanup: Killing windows"));
+	gdm_slave_windows_kill();
 
-    exit(DISPLAY_REMANAGE);
+	exit(DISPLAY_REMANAGE);
+    }
 }
 
 
@@ -457,7 +482,7 @@ gdm_slave_term_handler(int sig)
 	waitpid(d->greetpid, 0, 0); 
 	d->greetpid=0;
     } 
-    else if(pwent)		/* FIXME Maybe this is not enough */
+    else if(pwent)
 	gdm_slave_session_stop();
 
     gdm_debug(_("gdm_slave_term_handler: Whacking client connections"));
@@ -480,14 +505,13 @@ gdm_slave_child_handler(int sig)
 	if(WIFEXITED(status))
 	    gdm_debug(_("gdm_slave_child_handler: %d returned %d"), pid, WEXITSTATUS(status));
 
-	if(pid==d->greetpid) {
-	    gdm_slave_start(d);
-	};
+	if(pid==d->greetpid && greet)
+	    exit(DISPLAY_REMANAGE);
 	
 	if(pid==d->sesspid) {
 	    gdm_slave_session_stop();
 	    gdm_slave_session_cleanup();
-	};
+	}
     }
 }
 
@@ -498,37 +522,38 @@ gdm_slave_windows_kill(void)
 {
     Window root, parent, *children;
     gint child, screen, nchildren;
-    Display *dsp=NULL;
+    Display *disp=NULL;
 
-    dsp=XOpenDisplay(d->name);
+    disp=XOpenDisplay(d->name);
 
-    if(!dsp) {
+    if(!disp) {
 	gdm_debug(_("gdm_slave_windows_kill: Could not open display %s"), d->name);
 	return;
-    };
+    }
 
     gdm_debug(_("gdm_slave_windows_kill: Killing windows on %s"), d->name);
 
     gdm_putenv(g_strconcat("XAUTHORITY=", d->auth, NULL));
+    gdm_putenv(g_strconcat("DISPLAY=", d->name, NULL));
 
-    for(screen=0 ; screen<ScreenCount(dsp) ; screen++) {
+    for(screen=0 ; screen<ScreenCount(disp) ; screen++) {
 
 	nchildren=0;
 
-	while(XQueryTree(dsp, RootWindow(dsp, screen), &root, &parent,
+	while(XQueryTree(disp, RootWindow(disp, screen), &root, &parent,
 			 &children, &nchildren) && nchildren>0) {
 	    
 	    for(child=0 ; child<nchildren ; child++) {
 		gdm_debug(_("gdm_slave_windows_kill: Killing child 0x%x"), children[child]);
-		XKillClient(dsp, children[child]);
-	    };
+		XKillClient(disp, children[child]);
+	    }
 	    
 	    XFree(children);
-	};
+	}
 
-    };
+    }
 
-    XSync(dsp, FALSE);
+    XSync(disp, FALSE);
 }
  
 
@@ -537,22 +562,69 @@ static gint
 gdm_slave_xerror_handler(Display *disp, XErrorEvent *evt)
 {
     gdm_debug(_("gdm_slave_windows_kill_error_handler: Blam"));
-
-    return(0);
-};
+    pingack=FALSE;
+    return(TRUE);
+}
 
 
 /* We respond to fatal errors by restarting the display */
 static gint
 gdm_slave_xioerror_handler(Display *disp)
 {
-    if(pwent->pw_uid)
+    gdm_debug(_("gdm_slave_xioerror_handler: I/O error for display %s"), d->name);
+
+    if(pwent)
 	gdm_slave_session_stop();
 
-    gdm_remanage(_("gdm_slave_windows_kill_ioerror_handler: Fatal X error - Restarting Xserver for %s"), 
-		 d->name);
+    gdm_error(_("gdm_slave_windows_kill_ioerror_handler: Fatal X error - Restarting %s"), d->name);
+
+    exit(DISPLAY_RESERVER);
+}
+
+
+static void
+gdm_slave_xsync_handler(int sig)
+{
+    gdm_debug(_("gdm_slave_xsync_handler: Xping failed for display %s"), d->name);
+    pingack=FALSE;
+}
+
+
+static gboolean
+gdm_slave_xsync_ping(void)
+{
+    struct sigaction sigalarm;
+    sigset_t mask, omask;
+
+    gdm_debug(_("gdm_slave_xsync_ping: Pinging %s"), d->name);
+
+    pingack=TRUE;
+
+    XSetErrorHandler(gdm_slave_xerror_handler);
+    XSetIOErrorHandler(gdm_slave_xioerror_handler);
+
+    sigalarm.sa_handler = gdm_slave_xsync_handler;
+    sigalarm.sa_flags = 0;
+    sigemptyset(&sigalarm.sa_mask);
+    
+    if(sigaction(SIGALRM, &sigalarm, NULL) < 0)
+	gdm_abort(_("gdm_slave_xsync_ping: Error setting up ALARM signal handler"));
+
+    sigemptyset(&mask);
+    sigaddset(&mask, SIGALRM);
+    sigprocmask(SIG_UNBLOCK, &mask, &omask);
+
+    alarm(10);
+
+    XSync(d->dsp, TRUE);
+
+    alarm(0);
+    sigprocmask(SIG_SETMASK, &omask, NULL);
+
+    gdm_debug(_("gdm_slave_xsync_ping: Pinging %s returned %d"), d->name, pingack);
+
+    return(pingack);
+}
 
-    exit(DISPLAY_REMANAGE);
-};
 
 /* EOF */
