@@ -32,11 +32,16 @@
 #include <security/pam_appl.h>
 
 extern pam_handle_t *pamh;
+
 #endif /* HAVE_PAM */
 
+#ifdef HAVE_SHADOW
+#include <shadow.h>
+#endif /* HAVE_SHADOW */
+
 #include "gdm.h"
 
-static const gchar RCSid[]="$Id: gdmverify.c,v 1.9 1999/01/25 22:00:06 mkp Exp $";
+static const gchar RCSid[]="$Id: gdmverify.c,v 1.11 1999/02/24 22:25:49 mkp Exp $";
 
 extern void gdm_abort(const char*, ...);
 
@@ -146,11 +151,25 @@ gdm_verify_user (gchar *login, gchar *pwd, gchar *display)
 
 #else /* HAVE_PAM */
 
+    gchar *passwd;
     struct passwd *pwent;
 
+#ifdef HAVE_SHADOW
+    struct spwd *sp;
+#endif
+
     pwent = getpwnam(login);
 
-    if (!pwent || strcmp(crypt(pwd, pwent->pw_passwd), pwent->pw_passwd))
+    passwd = !pwent ? NULL : pwent->pw_passwd;
+
+#ifdef HAVE_SHADOW
+    sp = getspnam(login);
+    if (sp != NULL)
+        passwd = sp->sp_pwdp;
+    endspent();
+#endif /* HAVE_SHADOW */
+
+    if (!passwd || strcmp(crypt(pwd, passwd), passwd))
     {
 	syslog(LOG_ERR, _("Couldn't authenticate %s"), login);
 	return(0);
@@ -171,6 +190,10 @@ gdm_verify_cleanup (void)
     pam_close_session(pamh, 0);
     pam_end(pamh, PAM_SUCCESS);
     pamh=NULL;
+
+    /* Workaround to avoid gdm messages being logged as PAM_pwdb */
+    closelog();
+    openlog("gdm", LOG_PID, LOG_DAEMON);
 #endif /* HAVE_PAM */
 }
 
